#include "buffer_objects.h"

#include "common_operations.h"
#include "glo_external.h"

namespace glc
{
  CommonBasicBuffer::CommonBasicBuffer( GLuint buffer_type ): BufferType(buffer_type)
  {
    glGenBuffers(1, &BufferName);
    glBindBuffer(buffer_type, BufferName);
  }

  void CommonBasicBuffer::Bind()
  {
    glBindBuffer(BufferType, BufferName);
  }

  void CommonBasicBuffer::Unbind()
  {
    glBindBuffer(BufferType, 0);
  }

  CommonBasicBuffer::~CommonBasicBuffer()
  {
    glDeleteBuffers(1, &BufferName);
  }

  void * CommonBasicBuffer::BindMap( GLintptr offset, GLsizeiptr size )
  {
    Bind();
    return glMapBufferRange(
      BufferType, offset,	size,
      GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
  }

  void CommonBasicBuffer::Unmap()
  {
    glUnmapBuffer(BufferType);
  }

  GLint glGetIntegerValue(GLenum pname)
  {
    GLint val;
    glGetIntegerv(pname, &val);
    return val;
  }

  void CommonBasicBuffer::SetData(const GLvoid* ptr, GLsizeiptr size, GLenum hint)
  {
    //ASSERT_BUFFER_BOUND_IF_TARGET(GL_ARRAY_BUFFER);
    //ASSERT_BUFFER_BOUND_IF_TARGET(GL_ELEMENT_ARRAY_BUFFER);
#ifdef NSIGHT_DEBUGGER
    WITH_BINDING(this) {glBufferData(BufferType, size, ptr, GL_STATIC_DRAW);}
#else
    glNamedBufferDataEXT(BufferName, size, ptr, GL_STATIC_DRAW);
#endif
    //glBindBuffer(BufferType, 0);
  }

  CommonElementArrayBuffer::CommonElementArrayBuffer( std::vector<GLuint>& data, GLenum primType ) :
  CommonTypedBuffer<GLuint>(GL_ELEMENT_ARRAY_BUFFER, data),
    PrimitiveType(primType),
    IndicesCount(data.size())
  {
  }
  void CommonElementArrayBuffer::DrawInstanced( GLuint instanceCount)
  {
    glDrawElementsInstanced(
      PrimitiveType, IndicesCount, INDEX_TYPE, 0, instanceCount);
  }

};