#pragma once

#include "glwindows.h"
#include "tricks.h"
#ifndef SWIG
#include <glm/glm.hpp>
#include <memory>
#include <string>
#endif
#include "textures.h"
namespace glf{ class compiler; }
namespace glc 
{
  using glm::mat4;
  using glm::vec2;
  using glm::ivec2;
  using glm::vec3;

  class CommonShader;
  class CommonProgram;
  class CommonPipeline;
  class CommonPipelineBuilder;

  TYPEDEF_AUTO_PTR_TYPE(CommonShader);
  TYPEDEF_AUTO_PTR_TYPE(CommonProgram);
  TYPEDEF_AUTO_PTR_TYPE(CommonPipeline);
  TYPEDEF_AUTO_PTR_TYPE(CommonPipelineBuilder);


  class CommonShaderCompiler
  {
  public:
    CommonShaderCompiler();
    PCommonShader CreateVertexShader(std::string filename);
    PCommonShader CreateFragmentShader(std::string filename);
    std::shared_ptr<::glf::compiler> Compiler;
  };

  class CommonShader
  {
  public:
    friend class CommonProgram;
  private:
    GLenum ShaderUsageBit;
    GLuint ShaderName;
  public:
    CommonShader(GLuint shader_name, GLenum shader_usage_bit);  
  };

  class CommonProgram
  {
  protected:
  public:
    friend class CommonPipeline;
  private:
    GLuint ProgramName;
    GLenum ProgramStages; 
  public:
    CommonProgram();
    ~CommonProgram();
    void InstallShader(CommonShader& shader);
    void Link();
    chain_reference<CommonProgram, CommonShader&> operator<< (CommonShader& sh);

    GLuint getUniformLocation(const char* name);
    void setUniform(GLuint loc, float val);
    void setUniform(GLuint loc, vec2 val);
    void setUniform(GLuint loc, ivec2 val);
    void setUniform(GLuint loc, vec3 val);
    void setUniform(GLuint loc, mat4 val);
    
    void BindTexture( GLuint binding, TextureNameReference tex );

    // introspection

    struct ResourceInfo
    {
      std::string name;
      GLenum type;
      GLint size;
      GLint location;
    };

    GLint GetActiveAttributesCount();
    GLint GetActiveUniformCount();
    GLint GetActiveResourceCount(GLenum inteface);
    ResourceInfo GetActiveAttrib(GLuint index);
    ResourceInfo GetActiveUnifrom(GLuint index); 
    // the properties : http://www.opengl.org/wiki/GLAPI/glGetProgramResource
    GLint GetResourceProp(GLenum inteface, GLint index, GLenum prop);
    std::string GetResourceName(GLenum interface, GLint index);
    GLint getUniformi( std::string name );
  };

  class VertexFragmentProgram: public CommonProgram
  {
   public:
    VertexFragmentProgram(std::string frag_shader, std::string vert_shader);
 
  };

  class TextureNameReference;
  class CommonPipeline
  {
    GLuint PipelineName;
  public:
    CommonPipeline();
    ~CommonPipeline();
    void InstallProgram(const CommonProgram&);
    void Bind();
    void Unbind();
  };

}


