﻿#include "textures.h"
#include "common.hpp"
#include "glo_external.h"
#include <gl/gl.h>


namespace glc
{
  std::stack<GLuint> TextureNameReference::bound;

 /* void TextureNameReference::PushBindZeroUnit()
  {
    glActiveTexture(GL_TEXTURE0);
    bound.push(this->Name);
    glBindTexture(GL_TEXTURE_2D, this->Name);
  }
  void TextureNameReference::PopBindZeroUnit()
  {
    glActiveTexture(GL_TEXTURE0);
    GLuint id = 0;
    assert(!bound.empty());
    assert(bound.top() == this->Name);
    
    bound.pop();
    if (!bound.empty())
      id = bound.top();

    glBindTexture(GL_TEXTURE_2D, id);
  }
*/

  ImageTexture::ImageTexture(GLsizei w, GLsizei h) : width(w), height(h) 
  {
    const int MAX_COLOR_DEPTH_BYTE = 4;
    buffer = (unsigned char*) malloc(width*height*MAX_COLOR_DEPTH_BYTE);
    SetCommonTextureParam();
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
  }

  void ImageTexture::LoadRGB(unsigned char* data)
  {
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    const int RGB_SIZE = 3;
   

    glTextureImage2DEXT(
      TextureName,
      GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, 
      CopyRowReverseBuffer(RGB_SIZE, data));
    
    glGenerateTextureMipmapEXT(TextureName, GL_TEXTURE_2D);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    
  }

  void ImageTexture::LoadFloats(const float* data )
  {
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    //float *f = (float* )data;
    //for (int i =0; i < width*height; i++)
    //  f[i] = 1.2;
    //std::cout << data[i] << std::endl;

    const int FLOAT_SIZE = 4;
    //SetCommonTextureParam();
    auto buf = CopyRowReverseBuffer(FLOAT_SIZE, (unsigned char*) data);
#ifdef NSIGHT_DEBUGGER
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, TextureName);
    glTexImage2D(
      GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT32F, width, height, 0, GL_DEPTH_COMPONENT, GL_FLOAT, buf);
    TextureName.SetParameter(GL_TEXTURE_COMPARE_MODE, GL_NONE);
    glBindTexture(GL_TEXTURE_2D, 0);
#else
      glTextureImage2DEXT(TextureName, 
        GL_TEXTURE_2D, 0, GL_R32F, width, height, 0, GL_RED, GL_FLOAT, buf);
      TextureName.SetParameter(GL_TEXTURE_COMPARE_MODE, GL_NONE);
#endif
    glGenerateTextureMipmapEXT(TextureName, GL_TEXTURE_2D);
    //glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

  }

  ImageTexture::~ImageTexture()
  {
    if (buffer)
      free(buffer);
  }

  unsigned char* ImageTexture::CopyRowReverseBuffer( const int color_depth, unsigned char* data )
  {
    size_t row_size = color_depth*width;
    for (int i=0; i<height; i++)
      memcpy(
      buffer+(height - i - 1) * row_size, 
      data + i*row_size, 
      row_size);
    return buffer;
  }

  void ImageTexture::SetCommonTextureParam()
  {
    TextureName.SetParameter(GL_TEXTURE_WRAP_S, GL_REPEAT);
    TextureName.SetParameter(GL_TEXTURE_WRAP_T, GL_REPEAT);
    TextureName.SetParameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    TextureName.SetParameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  }


  std::map<GLuint, unsigned> TextureNameReference::RefCount;

  void TextureNameReference::DecRef()
  {
    if(--RefCount[Name] == 0)
    {
      RefCount.erase(Name);
      glDeleteTextures(1, &Name); 
    }
  }

  void TextureNameReference::IncRef()
  {
    RefCount[Name] = RefCount[Name] + 1;
  }

  TextureNameReference::TextureNameReference()
  {
    glGenTextures(1, &Name);
    RefCount[Name] = 1;
  }

  TextureNameReference::TextureNameReference( const TextureNameReference& n )
  {
    assert(RefCount.count(n.Name));
    Name = n.Name;
    IncRef();
  }

  TextureNameReference& TextureNameReference::operator=( const TextureNameReference & src )
  {
    if (&src == this)
      return *this;
    DecRef();
    Name = src.Name;
    IncRef();
    return *this;
  }

  TextureNameReference::~TextureNameReference()
  {
    DecRef();
  }

  void TextureNameReference::SetParameter( GLuint name, GLuint val )
  {
    glTextureParameteriEXT(this->Name, GL_TEXTURE_2D,  name, val);
  }

};