#pragma once 
#include "glwindows.h"
#ifndef SWIG
#include <memory>
#endif

#define TYPEDEF_AUTO_PTR_TYPE(TYPE_NAME)\
  typedef std::shared_ptr<TYPE_NAME> P##TYPE_NAME;

namespace glc 
{
  template <typename T>
    std::shared_ptr<T> make_auto_ptr(T* o) {return std::shared_ptr<T>(o); }



#pragma region binding scope tricks

  template <typename T>
  class BindingScope
  {
    T BoundObject;
  public:
    bool InScope;
    BindingScope(T obj) : 
    BoundObject(obj), InScope(true)
    {    
      BoundObject->Bind();
    }
    ~BindingScope()
    {
      BoundObject->Unbind();
    }
  };

  class ScopedEnabling
  {
    GLenum Feature;
  public:
    bool InScope;
    ScopedEnabling(GLenum f) : Feature(f), InScope(true) { glEnable(Feature); }
    ~ScopedEnabling() { glDisable(Feature); }
  };

  /// handle auto_Ptr and ptr here
  template<typename T> 
  BindingScope<T&> make_binding(T& o)
  {
    return BindingScope<T&>(o);
  }
  template<typename T> 
  BindingScope<T*> make_binding(T* o)
  {
    return BindingScope<T*>(o);
  }
#define USE_BINDING_SCOPE(OBJ, SCOPE_CONSTRUCTOR)\
  for (auto\
  MACRO_BINDING_CONTEXT = SCOPE_CONSTRUCTOR(OBJ);\
  MACRO_BINDING_CONTEXT.InScope;\
  MACRO_BINDING_CONTEXT.InScope = false)

#define WITH_BINDING(OBJ) USE_BINDING_SCOPE(OBJ, make_binding)
#define WITH_ENABLE(OBJ) USE_BINDING_SCOPE(OBJ, ScopedEnabling)

#pragma endregion


  template <class THost, class TArg>
  class chain_reference
  {
    THost& host;
  public:
    chain_reference(THost& h) : host(h) { }
    chain_reference<THost, TArg> & operator << (TArg a){ return host << a; }
  };

}