#include "vertex_arrays.h"
#include "glo_external.h"
#include <assert.h>


namespace glc{

  void CommonVertexArray::Bind()
  {
    bind_stack.push_back(VertexArrayName);
    glBindVertexArray(VertexArrayName);
  }

  void CommonVertexArray::Unbind()
  {
    assert(bind_stack.back() == VertexArrayName);
    bind_stack.pop_back();
    GLuint top = bind_stack.empty() ? 0 : bind_stack.back();
    glBindVertexArray(top);
  }

  CommonVertexArray::CommonVertexArray()
  {
    glGenVertexArrays(1, &VertexArrayName);
  }

  CommonVertexArray::~CommonVertexArray()
  {
    glDeleteVertexArrays(1, &VertexArrayName);
  }

  CommonVertexAttribute CommonVertexArray::BindAttribute( GLuint index ) 
  {
    CommonVertexAttribute a = CommonVertexAttribute(index);
#ifdef NSIGHT_DEBUGGER 
    WITH_BINDING(this) { glEnableVertexAttribArray(index);}
#else
    glEnableVertexArrayAttribEXT(VertexArrayName, index);
#endif
    return a;
  }


  void CommonVertexArray::CheckBound()
  {
    assert(!bind_stack.empty() && bind_stack.back() == VertexArrayName);
  }

  void CommonVertexArray::BindElementBuffer( CommonElementArrayBuffer & buf )
  {
    CheckBound();
    buf.Bind();
  }

  void CommonVertexArray::DrawInstanced(
    GLsizei element_count,  GLsizei instance_count, GLenum mode)
  {
    CheckBound();
    glDrawArraysInstanced(mode, 0, element_count, instance_count);
  }

  void CommonVertexArray::SpecifyAttribute( PCommonBasicBuffer buff, CommonVertexAttribute atr, AttributeSpec spec )
  {
#ifdef NSIGHT_DEBUGGER
    WITH_BINDING(this)
    {
      WITH_BINDING(buff) {
        glVertexAttribPointer(
          atr.Index, spec.Size, spec.Type, GL_FALSE, spec.Stride, GLF_BUFFER_OFFSET(spec.Offset));
      }
    }
#else
    glVertexArrayVertexAttribOffsetEXT(
      VertexArrayName, *buff, 
      atr.Index, spec.Size, spec.Type, GL_FALSE, spec.Stride, (GLintptr)GLF_BUFFER_OFFSET(spec.Offset));
#endif
  }

  std::vector<GLuint> CommonVertexArray::bind_stack;

  CommonVertexAttribute::CommonVertexAttribute(GLuint index):
  Index(index)
  {
  }
 

};