#include "CameraModule.h"
#include "converters.h"
#include <stdexcept>

 

CameraModule::CameraModule( const Camera* c ) : camera(c) ,init(false)
{
}

glm::mat4 CameraModule::GetCurrentMVP()
{
	if (!init)
	{
		std::cout << "camera parameters not set";
		throw std::runtime_error("camera parameters not set");
	}
  glm::mat4 Projection =  Proj;
  glm::mat4 Rot = camera->GetCurrentRotation();
  glm::mat4 Trans = camera->GetCurrentTranslation();
  return Projection*Trans*Model*Rot;
}


void CameraModule::AssignCameraParameters( CameraAcelAcessor cam)
{
  position = cam.position();
  direction = cam.direction();
  up = cam.up();

  Model = GetCameraLookAtMatrix(cam);
  Proj =  GetCameraProjectionMatrix(cam); 

  init = true;
}

glm::mat4 CameraModule::GetDoFMVP(float bokeh_x, float bokeh_y, float focal_length, float aperture)
{
  if (!init)
  {
    std::cout << "camera parameters not set";
    throw std::runtime_error("camera parameters not set");
  }

	glm::vec3 object = position + direction * focal_length;
	glm::vec3 right = glm::normalize(glm::cross(direction, up));
	//glm::vec3 p_up = glm::normalize(glm::cross(object - eye, right));

  glm::vec3 bokeh = right * bokeh_x +  up * bokeh_y;
	glm::mat4 modelview = glm::lookAt(position + aperture * bokeh, object, up);
	return Proj * modelview;
}

glm::mat4 GetCameraProjectionMatrix( const CameraAcelAcessor& c )
{
  using namespace glm;
  float aspect = c.aspect_ratio();
  float xfov = c.xfov();
  float yfov = degrees(2 * atan( tan(radians(xfov / 2)) /aspect) );

  return perspective<float>(
    yfov,
    c.aspect_ratio(), 
    c.znear(), 
    c.zfar()) ;
}

glm::mat4 GetCameraLookAtMatrix( const CameraAcelAcessor &c )
{
  return glm::lookAt(c.position(), c.position()+ c.direction(), c.up());
}
