#include "DepthRenderingModule.h"
#include "shader_modules/depth.meta.h"
#include "renderer-core/common_operations.h"
#include <scene-core/sra_implementation.h>
#include "converters.h"
#include "Renderer.h"

using namespace glc;
using demo::Renderer;

extern float REN_DEPTH_TRESHOLD;


DepthMapRenderingModule::DepthMapRenderingModule(int w, int h) : win_size(w, h)
{
  Array = std::make_shared<ShaderProgramdepth_VertexArray>();
  SetWinSize(win_size);
  SetMaxDepthGap(REN_DEPTH_TRESHOLD);
}

void DepthMapRenderingModule::AssignCameraParameters(CameraAcelAcessor cam)
{
  //  move this to camera translator
  using namespace glm;
  float aspect = cam.aspect_ratio();
  auto tgFovX= glm::tan(radians(cam.xfov())/2.0 );
  auto tgFovY = glm::tan( atan( tan(radians(cam.xfov() / 2)) /aspect) );

  SetTgFov(glm::vec2(tgFovX, tgFovY));
  SetDepthCameraToWorld(inverse(GetCameraLookAtMatrix(cam)));
}


void DepthMapRenderingModule::Draw()
{
  WITH_BINDING(&Pipeline)
  {
    WITH_BINDING(Array)
    {
      Array->DrawInstanced(6, win_size.x * win_size.y);
    }
  }
}

void DepthMapRenderingModule::AssignWorldToLight( glm::mat4 m )
{

  glm::mat4 clip_to_tex(
    0.5, 0.0, 0.0, 0.0,
    0.0, 0.5, 0.0, 0.0,
    0.0, 0.0, 0.5, 0.0,
    0.5, 0.5, 0.5, 1.0
    );
  SetWorldToLightClip(clip_to_tex*m);
}

 