#include "Renderer.h"

#include "renderer-core/glo_external.h"
#include "renderer-core/window.h"
#include "renderer-core/tricks.h"
#include "renderer-core/fbo.h"
#include "renderer-core/opengl_version.h"



using glf::window;

extern bool REN_USE_OPENGL_42;
extern bool REN_FULL_SCREEN;
extern bool ORANGE_BACKGROUND;

using namespace demo;
void Renderer::ClearBuffer()
{
  float Depth(1.0f);
  glClearBufferfv(GL_DEPTH , 0, &Depth);
  //glClearColor(1.0f, 0.5f, 0.0f, 1.0f);
  if (ORANGE_BACKGROUND)
	glClearBufferfv(GL_COLOR, 0, &glm::vec4(1.0f, 0.5f, 0.0f, 1.0f)[0]);
  else
	glClearBufferfv(GL_COLOR, 0, &glm::vec4(0.0f, 0.0f, 0.0f, 0.0f)[0]);

  //glClearDepth(10.f);
}

void Renderer::BeginFrame()
{
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
  glViewportIndexedf(0, 0, 0, GLfloat(width), GLfloat(height));
  ClearBuffer();
}

Renderer::Renderer( int width, int height ) : width(width), height(height), camera(0) 
{
  glf::GLInfo  v;
  if (REN_USE_OPENGL_42)
    v = glf::GLInfo(4, 2, REN_FULL_SCREEN);
  else
    v = glf::GLInfo(3, 3, REN_FULL_SCREEN);

  window::Open(glm::ivec2(width, height), "rendering demo", v );
  
  glClampColor(GL_CLAMP_READ_COLOR, GL_FALSE);

  camera = Camera(window::Instance());
  window::Instance()->EnableDebug();
}

void Renderer::EndFrame()
{
  glDisable(GL_DEPTH_TEST);
  window::Instance()->SwapBuffers();
}

const Camera* Renderer::GetCamera()
{
  return &camera;
}

int Renderer::Height()
{
  return height;
}

int Renderer::Width()
{
  return width;
}

std::shared_ptr<glc::CommonFBO> Renderer::CreateFBO(bool shadowmode)
{
  return glc::make_auto_ptr(new glc::CommonFBO(Width(), Height(), shadowmode) );
}
std::shared_ptr<glc::DepthFBO> Renderer::CreateDepthFrameBuffer(bool shadowmode)
{
  return glc::make_auto_ptr(new glc::DepthFBO(Width(), Height(), shadowmode) );
}
bool Renderer::Closing()
{
  return window::Instance()->Closing();
}

#include <GL/glfw.h>
bool Renderer::Paused()
{
  return window::Instance()->KeyPressed[GLFW_KEY_SPACE];
}


 

 