#pragma once
#include <vector>
#include <string>
#include <glm/glm.hpp>
#include <glm/ext.hpp>
#include <scene-core/sra-api/sra_scene.h>
#include <renderer-demo/type_trans.h>

class GeometryAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  GeometryAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  int id() const {  return convert(acel->Property("id")->Value()->GetInt()); }
  sra::float_buffer_t vertices() const {  return convert(acel->Property("vertices")->Value()->GetFloatBuffer()); }
  sra::float_buffer_t normals() const {  return convert(acel->Property("normals")->Value()->GetFloatBuffer()); }
  sra::float_buffer_t texcoord() const {  return convert(acel->Property("texcoord")->Value()->GetFloatBuffer()); }
  sra::int_buffer_t indices() const {  return convert(acel->Property("indices")->Value()->GetIntBuffer()); }
  int texture_id() const {  return convert(acel->Property("texture_id")->Value()->GetInt()); }
};
class VectorAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  VectorAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  float x() const {  return convert(acel->Property("x")->Value()->GetFloat()); }
  float y() const {  return convert(acel->Property("y")->Value()->GetFloat()); }
  float z() const {  return convert(acel->Property("z")->Value()->GetFloat()); }
};
class TextureDepthAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  TextureDepthAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  sra::float_buffer_t float_values() const {  return convert(acel->Property("float_values")->Value()->GetFloatBuffer()); }
  int width() const {  return convert(acel->Property("width")->Value()->GetInt()); }
  int height() const {  return convert(acel->Property("height")->Value()->GetInt()); }
};
class TextureRGBAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  TextureRGBAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  int id() const {  return convert(acel->Property("id")->Value()->GetInt()); }
  sra::byte_buffer_t values() const {  return convert(acel->Property("values")->Value()->GetByteBuffer()); }
  int width() const {  return convert(acel->Property("width")->Value()->GetInt()); }
  int height() const {  return convert(acel->Property("height")->Value()->GetInt()); }
};
class CameraAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  CameraAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  float xfov() const {  return convert(acel->Property("xfov")->Value()->GetFloat()); }
  float aspect_ratio() const {  return convert(acel->Property("aspect_ratio")->Value()->GetFloat()); }
  float znear() const {  return convert(acel->Property("znear")->Value()->GetFloat()); }
  float zfar() const {  return convert(acel->Property("zfar")->Value()->GetFloat()); }
  glm::vec3 position() const {  return convert(acel->Property("position")->Value()->GetFloat3()); }
  glm::vec3 direction() const {  return convert(acel->Property("direction")->Value()->GetFloat3()); }
  glm::vec3 up() const {  return convert(acel->Property("up")->Value()->GetFloat3()); }
};
class SpotLightAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  SpotLightAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  float falloff_angle() const {  return convert(acel->Property("falloff_angle")->Value()->GetFloat()); }
  glm::vec3 position() const {  return convert(acel->Property("position")->Value()->GetFloat3()); }
  glm::vec3 direction() const {  return convert(acel->Property("direction")->Value()->GetFloat3()); }
  glm::vec3 up() const {  return convert(acel->Property("up")->Value()->GetFloat3()); }
};

