#include "converters.h"
#include <scene-core/sra_implementation.h>
#include <iostream>

using std::shared_ptr;

shared_ptr<ShaderProgrambasic_VertexArray>  CreateMesh( GeometryAcelAcessor g )
{
	using namespace std;
	auto m =  make_shared<ShaderProgrambasic_VertexArray>();
	assert( sizeof(GLuint) == sizeof(int32_t));
	auto foo = g.indices();
	vector<GLuint> elem(
		(GLuint*) g.indices().Ptr(),
		(GLuint*)(g.indices().Ptr()) +  g.indices().Size());

	m->SetTriangleElements(elem);

	m->SetAttributePosition(vector<float>( g.vertices().Ptr(), g.vertices().Ptr() + g.vertices().Size())) ;
	m->SetAttributeTexCoord(vector<float>( g.texcoord().Ptr(), g.texcoord().Ptr() + g.texcoord().Size())) ;
	m->SetAttributeNormal(vector<float>(  g.normals().Ptr(), g.normals().Ptr() + g.normals().Size())) ;
	return m;
}

void printmatrix( glm::mat4& m )
{
	std::cout 
		<< m[0][0] 
	<< m[0][1] 
	<< m[0][2] 
	<< m[0][3] 

	<< m[1][0] 
	<< m[1][1] 
	<< m[1][2] 
	<< m[1][3] 

	<< m[2][0] 
	<< m[2][1] 
	<< m[2][2] 
	<< m[2][3] 

	<< m[3][0] 
	<< m[3][1] 
	<< m[3][2] 
	<< m[3][3] ;
}

using glc::ImageTexture;
std::shared_ptr<ImageTexture> CreateTexture( TextureRGBAcelAcessor txt )
{
	auto res = std::make_shared<ImageTexture>(txt.width(), txt.height());
	/*for (int i=0; i<txt.values().Size(); i++)
		std::cout << txt.values().Ptr()[i];*/
	res->LoadRGB((unsigned char*)txt.values().Ptr());
	return res;
}

using glc::ImageTexture;

std::vector<float> ReadDepthTexture( std::shared_ptr<sra::IAcel> a)
{
  //auto res = std::make_shared<ImageTexture>(txt.width(), txt.height());
  auto data = a->Property(XSTREAM_FRAME_DEPTH_DATA)->Value()->GetFloatBuffer();
  std::vector<float> r(data.Ptr(), data.Ptr() + data.Size());
  return r;
}

glm::mat4 CreateLightPOV( CameraAcelAcessor cam, SpotLightAcelAcessor light )
{
  glm::mat4 proj = glm::perspective<float>(
    light.falloff_angle(),
    1.0, 
    cam.znear(), 
    cam.zfar()) ;

  glm::mat4 model = glm::lookAt(light.position(), light.position()+ light.direction(), light.up());
  return proj * model;
  

}

 


