#pragma once

#ifndef SWIG
#include <memory>
#endif

#include <scene-core/sra-api/sra_modules.h>

class CameraModule;
class MeshRenderingModule;
class DepthMapRenderingModule;
class SplashDrawModule;
class ShadowComputationModule;

namespace sra_render 
{
  class ModViewPoint : public sra::IModule
  {
    std::shared_ptr<CameraModule> module;
  public:
    ModViewPoint(std::shared_ptr<CameraModule> c);
    std::shared_ptr<sra::IConverter> CameraSettings;
    virtual std::shared_ptr<sra::IAcel> Execute();
  };
   
  class ModRenderMesh : public sra::IModule
  {
    std::shared_ptr<MeshRenderingModule> module;
  public:
    ModRenderMesh(std::shared_ptr<MeshRenderingModule> module);
    std::shared_ptr<sra::IConverter> Transform;
    std::shared_ptr<sra::IConverter> Texture;
    std::shared_ptr<sra::IConverter> Mesh;

    std::shared_ptr<sra::IConverter> ShadowTransform;
    std::shared_ptr<sra::IConverter> ShadowMap;
    virtual std::shared_ptr<sra::IAcel> Execute();
  };

  class ModRenderDepthMap : public sra::IModule
  {
    std::shared_ptr<DepthMapRenderingModule> module;
  public:
    ModRenderDepthMap(std::shared_ptr<DepthMapRenderingModule> module);
    std::shared_ptr<sra::IConverter> Transform;
    std::shared_ptr<sra::IConverter> Texture;
    std::shared_ptr<sra::IConverter> DepthMap;
    std::shared_ptr<sra::IConverter> CameraSettings;

    std::shared_ptr<sra::IConverter> ShadowMap;
    std::shared_ptr<sra::IConverter> ShadowTransform;
    virtual std::shared_ptr<sra::IAcel> Execute();
  };

  class ModDrawSplash : public sra::IModule
  {
    std::shared_ptr<SplashDrawModule> module;
  public:
    ModDrawSplash(std::shared_ptr<SplashDrawModule> module);
    std::shared_ptr<sra::IConverter> Texture;
    virtual std::shared_ptr<sra::IAcel> Execute();
  };

  
  class ModComputeShadows : public sra::IContextModule
  {
    std::shared_ptr<ShadowComputationModule> module;
  public:
    ModComputeShadows (std::shared_ptr<ShadowComputationModule> module);
    virtual std::shared_ptr<sra::IAcel> Execute();
    virtual void Begin();
    virtual void End();
  };

}