#include "sra_renderer.h"
#include "AcelData.h"
#include "StreamConverters.h"
#include "DepthRenderingModule.h"
#include "CameraModule.h"
#include "DepthRenderingModule.h"
#include "MeshRenderingModule.h"
#include "MeshesConverter.h"
#include "SplashDrawModule.h"



class BasicTest
{
  std::shared_ptr<AcelData> data;
  std::shared_ptr<Renderer> renderer;
  std::shared_ptr<CameraModule> mod_camera;
  std::shared_ptr<MeshRenderingModule> mod_render_mesh;
  std::shared_ptr<DepthMapRenderingModule> mod_render_depth;
  std::shared_ptr<SplashDrawModule> mod_draw_splash;
  std::shared_ptr<ImageStreamConverter> images;
  std::shared_ptr<DepthStreamConverter> depth;
  std::shared_ptr<  MeshesConverter > meshes;
  std::shared_ptr<DepthFBO> fbo;
  PImageTexture main_mesh_texture;
  glm::mat4 light_mvp;
public:
  glm::mat4 GetMVP() {return mod_camera->GetCurrentMVP(); }
  bool show_meshes;
  bool show_depth;
  bool show_shadows;
//  PImageTexture current_depth;
  //PImageTexture current_image;
  BasicTest(string s);
  void RenderMeshesFrame( PImageTexture im, glm::mat4 mvp);
  void RenderDepthFrame(glm::mat4 mvp);
  void RenderStreamFrame(std::shared_ptr<sra::IStream>);

  void render_default();
  void render_depth();
  void render_splash();
  void render_mesh();

  PImageTexture GetMeshTexture( );

  void compute_shadows(glm::mat4 mvp);
  template <typename T>
  void RenderInALoop(T func)
  {
    while(!renderer->Closing())
    {
      renderer->BeginFrame();
      if (!renderer->Paused())
      {
        data->UpdateStream();
      }
      auto mvp = mod_camera->GetCurrentMVP();
      func(mvp);
      renderer->EndFrame();
    }
  }
  void RenderDefaultFrame(std::shared_ptr<sra::IAcel>frame, glm::mat4 mvp);

};

BasicTest::BasicTest( string s ) : 
show_depth(false), show_meshes(false), show_shadows(true)
{

  data = make_auto_ptr( new AcelData(s));
  renderer = make_auto_ptr(new Renderer(REN_VIEWPORT_WIDTH, REN_VIEWPORT_HEIGHT) ) ;

  mod_camera = make_auto_ptr(new CameraModule(renderer->GetCamera()));
  mod_render_mesh = make_auto_ptr(new MeshRenderingModule());
  mod_draw_splash = make_auto_ptr(new SplashDrawModule(REN_VIEWPORT_WIDTH, REN_VIEWPORT_HEIGHT));

  mod_render_depth = make_auto_ptr(new DepthMapRenderingModule(renderer->Width(), renderer->Height()));
  mod_render_depth->AssignCameraParameters(data->GetCameraAcel());

  images = make_auto_ptr(new ImageStreamConverter());
  depth = make_auto_ptr(new DepthStreamConverter());
  meshes = make_auto_ptr(new MeshesConverter(mod_render_mesh));
  fbo = renderer->CreateDepthFrameBuffer();

  mod_camera->AssignCameraParameters(data->GetCameraAcel());
  main_mesh_texture = GetMeshTexture();
  light_mvp = mod_camera->GetCurrentMVP();

}

void BasicTest::RenderMeshesFrame( PImageTexture im, glm::mat4 mvp )
{
  if (im)
    mod_render_mesh->AssignTextureDiffuse(im);
  mod_render_mesh->SetMVP(mvp);
  glm::vec4 pos = glm::inverse(light_mvp) * vec4(0,0,0,1);
  mod_render_mesh->SetLight(glm::vec3(pos.x, pos.y, pos.z) );
  auto patches = data->PatchesAcel();
  auto patches_list = patches->List();
  for (int i=0; i < patches_list->Count(); i++)
  {
    meshes->Assign(patches_list->Get(i));
    mod_render_mesh->Draw();
  }
}

void BasicTest::render_depth()
{  
  RenderInALoop([this](glm::mat4 mvp) { 
    RenderDepthFrame(mvp); } );

}

void BasicTest::render_splash()
{
	ORANGE_BACKGROUND = false;
  // render depth to fbo
  while(!renderer->Closing())
  {
	 
    data->UpdateStream();
    renderer->BeginFrame();
	auto fbo = renderer->CreateFBO();
	 
	WITH_BINDING(fbo)
	{
		fbo->Clear();
		auto frame = data->stream->GetCurrentFrame();
		auto im = images->LoadTexture(frame); 
		mod_draw_splash->AssignTextureDiffuse(im);
		mod_draw_splash->Draw();
	}
	 
	mod_draw_splash->BindTextureDiffuse(fbo->ColorTexture);
	mod_draw_splash->Draw();
    renderer->EndFrame();
  }
}

void BasicTest::render_mesh()
{
  PImageTexture im = GetMeshTexture();
  compute_shadows(light_mvp);

  // render depth to fbo
  RenderInALoop([=](glm::mat4 mvp) { RenderMeshesFrame(im, mvp); } );
}

void BasicTest::render_default()
{
  RenderInALoop([=](glm::mat4 mvp) { RenderDefaultFrame(data->stream->GetCurrentFrame(), mvp); } );
}

void BasicTest::RenderDefaultFrame(std::shared_ptr<sra::IAcel>frame, glm::mat4 mvp)
{
  //current_depth = depth->LoadTexture(frame);
  //current_image = images->LoadTexture(frame);
 
  compute_shadows(light_mvp);
  if (show_depth)
    RenderDepthFrame( mvp);

  if (show_meshes)
    RenderMeshesFrame(main_mesh_texture, mvp);
}

void BasicTest::RenderStreamFrame(std::shared_ptr<sra::IStream>s)
{
  auto frame =s->GetCurrentFrame();
  auto current_depth = depth->LoadTexture(frame);
  auto current_image = images->LoadTexture(frame);
  mod_render_depth->AssignTextureDiffuse(current_image );
  mod_render_depth->AssignTextureDepth(current_depth);
  mod_render_depth->AssignCameraParameters(s->GetCamera());
  mod_render_depth->Draw();
}

void BasicTest::RenderDepthFrame( glm::mat4 mvp)
{ 
  mod_render_depth->SetMVP(mvp);
  
  RenderStreamFrame(data->stream);
  //RenderStreamFrame(data->stream3);
  //RenderStreamFrame(data->stream2);
}

PImageTexture BasicTest::GetMeshTexture( )
{
	return 0;
 //try {
  //  take data for the first frame
  data->UpdateStream();
  // need to take texture from different source
  auto frame = data->stream->GetCurrentFrame();
  auto im = images->LoadTexture(frame); 	

  return im;
  
}

void BasicTest::compute_shadows( glm::mat4 mvp )
{
  //if (!show_shadows)
  //  return;
  WITH_BINDING(fbo)
  {
    fbo->Clear();
    if (show_depth)
      RenderDepthFrame( mvp);
    if (show_meshes)
      RenderMeshesFrame(0, mvp);
  }

  glm::mat4 clip_to_tex(
    0.5, 0.0, 0.0, 0.0,
    0.0, 0.5, 0.0, 0.0,
    0.0, 0.0, 0.5, 0.0,
    0.5, 0.5, 0.5, 1.0
    );
  if (show_meshes) 
  {
    mod_render_mesh->BindTextureShadowMap(fbo->DepthTexture);
    mod_render_mesh->SetWorldToLightClip(clip_to_tex*mvp);
  }
  if (show_depth)
  {
    mod_render_depth->BindTextureShadowMap(fbo->DepthTexture);
    mod_render_depth->SetWorldToLightClip(clip_to_tex*mvp);
  }

}




int main()
{ 

  //BasicTest pool("../sample_data/twocubes/twocubes.scene");
  BasicTest pool("../../sample_data/small_hhi/small_hhi.scene");
  //buck.render_depth();

  //REN_VIEWPORT_WIDTH = 
  //REN_VIEWPORT_HEIGHT = 
  //REN_VIEWPORT_WIDTH = 494;
  //REN_VIEWPORT_HEIGHT = 404;

  //  BasicTest monkey("../sample_data/monkey.scene");
  //monkey.render_mesh();

  //BasicTest fall("../sample_data/fall.scene");
  //fall.show_depth = true;
  //fall.show_meshes = true;
  //fall.render_default();

  pool.show_shadows = false;
  pool.show_depth = false;
  pool.show_meshes = false;
	//auto mvp = pool.GetMVP(); 
  pool.render_splash();

  //render_mesh("../sample_data/monkey.scene");
  return 0;
}

