#pragma once
#include "renderer-core/common_operations.h"
#include "renderer-core/vertex_arrays.h"

#ifndef SWIG
#include <vector>
#include <string>
#include <memory>
#endif

extern std::string REN_SHADERS_PATH;

using glm::mat4;
using glm::vec2;
using glm::vec3;
using glm::vec4;
using glm::ivec2;

using glc::VertexFragmentProgram;
using glc::PImageTexture;
using glc::TextureNameReference;
using glc::CommonVertexArray;
using glc::CommonBasicBuffer;
using glc::CommonElementArrayBuffer;


class ShaderProgrambasic_VertexArray : public CommonVertexArray
{
  std::shared_ptr<CommonBasicBuffer> NormalAttrib;
  std::shared_ptr<CommonBasicBuffer> PositionAttrib;
  std::shared_ptr<CommonBasicBuffer> TexCoordAttrib;
  std::shared_ptr<CommonElementArrayBuffer> Elements;
public:
  void SetAttributeNormal(const std::vector<float> & val)
  {
    using namespace glc;
    auto type = make_float_attribute(3);
    unsigned int loc = 2;
    auto& atr = NormalAttrib = make_vertex_buffer(val);
    SpecifyAttribute(atr, BindAttribute(loc), type);    
  }
  void SetAttributePosition(const std::vector<float> & val)
  {
    using namespace glc;
    auto type = make_float_attribute(3);
    unsigned int loc = 1;
    auto& atr = PositionAttrib = make_vertex_buffer(val);
    SpecifyAttribute(atr, BindAttribute(loc), type);    
  }
  void SetAttributeTexCoord(const std::vector<float> & val)
  {
    using namespace glc;
    auto type = make_float_attribute(2);
    unsigned int loc = 3;
    auto& atr = TexCoordAttrib = make_vertex_buffer(val);
    SpecifyAttribute(atr, BindAttribute(loc), type);    
  }
  void SetTriangleElements(std::vector<GLuint> ind)  
  {
    Elements = glc::make_auto_ptr(new CommonElementArrayBuffer(ind, GL_TRIANGLES));
    WITH_BINDING(this) { BindElementBuffer(*Elements); }
  }
  void DrawTrianglesInstanced(GLuint instCount=1)
  {
    Elements->DrawInstanced(instCount);
  }
};

class ShaderProgrambasic : public VertexFragmentProgram
{ 
public:
  std::shared_ptr<ShaderProgrambasic_VertexArray> Array;
  void SetArray(std::shared_ptr<ShaderProgrambasic_VertexArray> a)  { Array = a; }
  virtual void DrawArray() 
  { 
     Array->Bind();
     Array->DrawTrianglesInstanced(); 
     Array->Unbind();
  }
  void SetLight(vec3 Light) { setUniform(getUniformLocation("Light"), Light); ; }
  void SetMVP(mat4 MVP) { setUniform(getUniformLocation("MVP"), MVP); ; }
  void SetWorldToLightClip(mat4 WorldToLightClip) { setUniform(getUniformLocation("WorldToLightClip"), WorldToLightClip); ; }
  void Setlambda(float lambda) { setUniform(getUniformLocation("lambda"), lambda); ; }
  
  void AssignTextureDiffuse(PImageTexture txt ) { BindTexture(1, txt->TextureName); } 
  void BindTextureDiffuse(TextureNameReference txt ) { BindTexture(1, txt); } 
  void AssignTextureShadowMap(PImageTexture txt ) { BindTexture(5, txt->TextureName); } 
  void BindTextureShadowMap(TextureNameReference txt ) { BindTexture(5, txt); } 

  ShaderProgrambasic() :
    VertexFragmentProgram(
      REN_SHADERS_PATH + "/shaders/basic.frag",
      REN_SHADERS_PATH + "/shaders/basic.vert")
  {
  }
};


