#pragma once
#include "renderer-core/common_operations.h"
#include "renderer-core/vertex_arrays.h"

#ifndef SWIG
#include <vector>
#include <string>
#include <memory>
#endif

extern std::string REN_SHADERS_PATH;

using glm::mat4;
using glm::vec2;
using glm::vec3;
using glm::vec4;
using glm::ivec2;

using glc::VertexFragmentProgram;
using glc::PImageTexture;
using glc::TextureNameReference;
using glc::CommonVertexArray;
using glc::CommonBasicBuffer;
using glc::CommonElementArrayBuffer;


class ShaderProgramdepth_VertexArray : public CommonVertexArray
{
  std::shared_ptr<CommonElementArrayBuffer> Elements;
public:
  void SetTriangleElements(std::vector<GLuint> ind)  
  {
    Elements = glc::make_auto_ptr(new CommonElementArrayBuffer(ind, GL_TRIANGLES));
    WITH_BINDING(this) { BindElementBuffer(*Elements); }
  }
  void DrawTrianglesInstanced(GLuint instCount=1)
  {
    Elements->DrawInstanced(instCount);
  }
};

class ShaderProgramdepth : public VertexFragmentProgram
{ 
public:
  std::shared_ptr<ShaderProgramdepth_VertexArray> Array;
  void SetArray(std::shared_ptr<ShaderProgramdepth_VertexArray> a)  { Array = a; }
  virtual void DrawArray() 
  { 
     Array->Bind();
     Array->DrawTrianglesInstanced(); 
     Array->Unbind();
  }
  void SetDepthCameraToWorld(mat4 DepthCameraToWorld) { setUniform(getUniformLocation("DepthCameraToWorld"), DepthCameraToWorld); ; }
  void SetMVP(mat4 MVP) { setUniform(getUniformLocation("MVP"), MVP); ; }
  void SetMaxDepthGap(float MaxDepthGap) { setUniform(getUniformLocation("MaxDepthGap"), MaxDepthGap); ; }
  void SetTgFov(vec2 TgFov) { setUniform(getUniformLocation("TgFov"), TgFov); ; }
  void SetWinSize(ivec2 WinSize) { setUniform(getUniformLocation("WinSize"), WinSize); ; }
  void SetWorldToLightClip(mat4 WorldToLightClip) { setUniform(getUniformLocation("WorldToLightClip"), WorldToLightClip); ; }
  
  void AssignTextureDepth(PImageTexture txt ) { BindTexture(2, txt->TextureName); } 
  void BindTextureDepth(TextureNameReference txt ) { BindTexture(2, txt); } 
  void AssignTextureDiffuse(PImageTexture txt ) { BindTexture(1, txt->TextureName); } 
  void BindTextureDiffuse(TextureNameReference txt ) { BindTexture(1, txt); } 
  void AssignTextureShadowMap(PImageTexture txt ) { BindTexture(5, txt->TextureName); } 
  void BindTextureShadowMap(TextureNameReference txt ) { BindTexture(5, txt); } 

  ShaderProgramdepth() :
    VertexFragmentProgram(
      REN_SHADERS_PATH + "/shaders/depth.frag",
      REN_SHADERS_PATH + "/shaders/depth.vert")
  {
  }
};


