#pragma once
#include "renderer-core/common_operations.h"
#include "renderer-core/vertex_arrays.h"

#ifndef SWIG
#include <vector>
#include <string>
#include <memory>
#endif

extern std::string REN_SHADERS_PATH;

using glm::mat4;
using glm::vec2;
using glm::vec3;
using glm::vec4;
using glm::ivec2;

using glc::VertexFragmentProgram;
using glc::PImageTexture;
using glc::TextureNameReference;
using glc::CommonVertexArray;
using glc::CommonBasicBuffer;
using glc::CommonElementArrayBuffer;


class ShaderProgramsdepth_VertexArray : public CommonVertexArray
{
  std::shared_ptr<CommonElementArrayBuffer> Elements;
public:
  void SetTriangleElements(std::vector<GLuint> ind)  
  {
    Elements = glc::make_auto_ptr(new CommonElementArrayBuffer(ind, GL_TRIANGLES));
    WITH_BINDING(this) { BindElementBuffer(*Elements); }
  }
  void DrawTrianglesInstanced(GLuint instCount=1)
  {
    Elements->DrawInstanced(instCount);
  }
};

class ShaderProgramsdepth : public VertexFragmentProgram
{ 
public:
  std::shared_ptr<ShaderProgramsdepth_VertexArray> Array;
  void SetArray(std::shared_ptr<ShaderProgramsdepth_VertexArray> a)  { Array = a; }
  virtual void DrawArray() 
  { 
     Array->Bind();
     Array->DrawTrianglesInstanced(); 
     Array->Unbind();
  }
  void SetZfar(float Zfar) { setUniform(getUniformLocation("Zfar"), Zfar); ; }
  void SetZnear(float Znear) { setUniform(getUniformLocation("Znear"), Znear); ; }
  void Setscreen_size(vec2 screen_size) { setUniform(getUniformLocation("screen_size"), screen_size); ; }
  
  void AssignTextureNDepth(PImageTexture txt ) { BindTexture(4, txt->TextureName); } 
  void BindTextureNDepth(TextureNameReference txt ) { BindTexture(4, txt); } 

  ShaderProgramsdepth() :
    VertexFragmentProgram(
      REN_SHADERS_PATH + "/shaders/splash-depth.frag",
      REN_SHADERS_PATH + "/shaders/fbo-splash.vert")
  {
  }
};


