#version 330 core

#define VERTEX_SHADER
#include "fbo-splash.glsl"
#include "texturing.h"
#include "util.h"
#include "screen_grid.h"
#include "unprojection.h"
#include "shadows.h"

uniform mat4 DepthCameraToWorld;

// put it into camera transform block
uniform mat4 MVP;
out gl_PerVertex  { vec4 gl_Position; };

uniform float MaxDepthGap;

void main()
{
  vec2 pixel = screen_grid_current_pixel();  
  vec2 pos_ndc;
  vec4 pos_eye;

  if (gl_VertexID == 1 || gl_VertexID == 4 )
  {
    triangle_ndc t_ndc  = get_current_triangle(pixel);
    triangle t_eye = triangle_ndc_to_eye(t_ndc);
    pos_ndc = t_ndc.a;
    pos_eye = t_eye.a;
    if (triangle_cutoff_z(t_eye, MaxDepthGap))
    {
      pos_eye = t_eye.b;
    }
  }
  else
  {
    pos_ndc =  screen_to_ndc(pixel + screen_grid_current_quad_vertex(0));
    pos_eye = ndc_to_eye(pos_ndc);
  }

  vec4 pos_world = DepthCameraToWorld * pos_eye;
  output_shadow(pos_world);
  gl_Position = MVP * pos_world;
  Texcoord = ndc_to_tex(pos_ndc);
}
