#pragma once
#ifndef SWIG
#include <string>
#include<memory>
#include <stdexcept>
#endif
#include <scene-core/sra-api/sra_modules.h>
#include <renderer-demo/public_modules.h>

extern std::string REN_SHADERS_PATH;
extern bool REN_FULL_SCREEN;
extern bool REN_USE_OPENGL_42;
extern bool ORANGE_BACKGROUND;

extern int REN_VIEWPORT_WIDTH;
extern int REN_VIEWPORT_HEIGHT;
extern float REN_DEPTH_TRESHOLD;



namespace sra_render
{
  class IDemoRenderer;

  /// entry point for the renderer implementation 
  /// (config is implementation specific string)
  /// renderer provides a collection of modules for processing acels
  std::shared_ptr<IDemoRenderer>  get_proto_renderer(std::shared_ptr<sra::IScene> scene, string config);

  // interface to access renderer functionality
  class IDemoRenderer
  {
  public:
    virtual bool Closing() = 0;
    virtual bool Paused() = 0;
    virtual void BeginFrame() = 0;
    virtual void EndFrame()= 0;
    
    // rendering modules can be created and used later
    // if the module doesn't exist UnsupportedModuleError is thrown
    virtual std::shared_ptr<ModRenderDepthMap> CreateModRenderDepthMap() = 0;
    virtual std::shared_ptr<ModRenderMesh> CreateModRenderMesh() = 0;
    virtual std::shared_ptr<ModViewPoint> CreateModViewPoint() = 0;
    virtual std::shared_ptr<ModDrawSplash> CreateModDrawSplash() = 0;
    virtual std::shared_ptr<ModComputeShadows> CreateModComputeShadows() = 0;
    virtual ~IDemoRenderer() {}
  }; 
  class UnsupportedModuleError : public std::exception {};

}