import path_loader
import os
from mako.template import Template



class ProgData():
  def __init__(self,  unif, smpl, atr, fields):
    self.simple_uniforms = unif
    self.samplers = smpl
    self.attributes = atr
    self.__dict__.update(fields)
    self.class_name = "ShaderProgram" + self.name 
    self.array_class_name = "Array" + self.name 
    if not 'fragment_shader_file'in fields:
      self.fragment_shader_file = self.name + ".frag"
    if not 'vertex_shader_file' in fields:
      self.vertex_shader_file = self.name + ".vert"

class Field():
  def __init__(self, name, type, location, type_token):    
    self.name = name
    self.type = type
    self.type_token = type_token
    self.location = location
  
  @property
  def cname(self):
    return self.name.replace(".",'_')
  
  @property  
  def qname(self):
    return "\"%s\"" % self.name


  @property
  def primtype(self):
    t = self.type_token[3:] #chomp "GL_"
    assert(t.find("MAT") == -1)
    assert(t.find("SAMPLER") == -1)
    dim = 1
    if t.find("VEC2") != -1:
      dim = 2;
    if t.find("VEC3") != -1:
      dim = 3;
    if t.find("VEC4") != -1:
      dim = 4;

    if t.startswith("FLOAT"):
      return "float", dim
    if t.startswith("INT"):
      return "int", dim
    if t.startswith("UNSIGNED_INT"):
      return "unsigned int", dim
    if t.startswith("BOOL"):
      return "bool", dim
    assert(false)
    return None


  @property
  def refvectype(self):     
    return "const std::vector<%s> &" % self.primtype[0]

  def gen_attribute_type_maker(self):
    return "make_%s_attribute(%d)" % self.primtype

  def gen_uniform_set(self):
    return \
     ("setUniform(getUniformLocation(%(qn)s), %(cn)s); ")\
      %     {"qn":self.qname, "cn":self.cname, "l":self.location}
    #return "setUniform(%d, %s)" % (self.location, self.name)
 
   

from shmet import read_shader_interface

def parse_shader_info(meta):
  uniforms = []
  samplers = []
  for u in meta.uniforms:
    if u.HasField('binding'):
      samplers.append(u) 
    else:
      uniforms.append(u)

  from glsl_typemap import typemap
  uniforms =   [Field(u.name, typemap[u.type], u.location, u.type) for u in uniforms]
  attributes = [Field(u.name, typemap[u.type], u.location, u.type) for u in meta.attributes]
  attributes = filter(lambda x: x.location != -1, attributes)
  return (uniforms, samplers, attributes)



from mako.lookup import TemplateLookup
import os.path

mylookup = TemplateLookup(directories=[os.path.abspath('.')])
mytemplate = Template(filename='tmpl.mako', lookup=mylookup )

def generate_classes(source, target, template_args):
  meta = read_shader_interface(source)
  u, s, a = parse_shader_info(meta)

  with open(target, 'wb') as f:
    f.write(mytemplate.render(prog=ProgData(u, s, a, template_args)))


if __name__ == "__main__":
  generate_classes("shader_meta/basic.meta", "test.h", 	dict(name = "basic" ))















