import scene 
from util_testing import image_check, image_save, fbo_check
from util_framebuffer import color_frame_buffer, depth_frame_buffer,  depth_frame_buffer_sampled
from util_scene import DemoScene

from util_rendering import RenderMeshes, RenderDepth, RenderSplash, RenderDof, new_demo_camera, DemoRenderer
from util_rendering import in_frame, frame_averaging, no_background
from util_rendering import with_depth_of_field, DepthOfField, RenderDepthToColor
from scene import cvar
cvar.REN_FULL_SCREEN = False

main_module =  __name__ == "__main__"
renderer = None

#renderer = scene.Renderer(1280, 720)

def screen_size():
  return (renderer.Width(), renderer.Height())

scene_buck_path =     "../../sample_data/buck/buck.scene"
scene_hhi_path  =     "../../sample_data/small_hhi/small_hhi.scene"
scene_twocubes_path = "../../sample_data/twocubes/twocubes.scene"
scene_girl_path =     "../../sample_data/man/girl.scene"
scene_surrey   =      "../../sample_data/sample_data_surrey/surrey2.scene"
#scene_mangeom = "../../sample_data/man/mannequin.scene"
scene_mangeom = scene_girl_path
 



def check_splash_shader(image_file, fbo, image):
  splash = RenderSplash(screen_size())
  with image_check(image_file, fbo):
    splash.draw(image)

def check_depth_shader(image_file, fbo, cam, data):
  depthren = RenderDepth(cam.camera_settings, screen_size())
  (image, depth)  = first_frame(data)

  mvp = cam.GetCurrentMVP() 
  with image_check(image_file, fbo):
     depthren.draw(mvp, depth, image)

def check_mesh_shader(image_file, fbo, cam, data, textures=None):
    meshren = RenderMeshes() 
    patches = data.list_patches()
    mvp = cam.GetCurrentMVP()
    with image_check(image_file, fbo):
       meshren.draw(mvp,  patches, textures)

def check_geom_texture_splash(scene_path, test_file ):
    data = DemoScene(scene_path)
    import scene
    txt = data.dict_textures[1]
    t = scene.CreateTexture(txt)
    check_splash_shader(
      test_file,
      color_frame_buffer(renderer),
      t)  

def check_meshes_textured(scene_path, test_file):
      camdata = default_camdata(scene_path)
      check_mesh_shader( 
      test_file,
      color_frame_buffer(renderer),
      *camdata,
      textures=camdata[1].dict_textures)  

def default_camdata(scene_file):
  data =  DemoScene(scene_file)
  return new_demo_camera(renderer, data), data

def first_frame(data):
    s = data.streams[0]
    s.next_frame()
    return s.get_image_frame(), s.get_depth_frame()

# def check_image(name, fbo):
#     splash = RenderSplash(screen_size())      
#     with image_check(name, color_frame_buffer(renderer)):
#       splash.draw(fbo.ColorTexture)


def render_depth_texture(draw, cam):  
  fbo_image = depth_frame_buffer_sampled(renderer) 
  with fbo_image:
      with in_frame(renderer):                   
          draw()

  dc = RenderDepthToColor(screen_size())
  fbo_res = color_frame_buffer(renderer)
  with fbo_res:
       dc.draw(fbo_image.DepthTexture, cam.camera_settings)
  return fbo_res


def render_depth_to_fbo(cam, data):
  depthren = RenderDepth(cam.camera_settings, screen_size())
  (image, depth)  = first_frame(data)
    
  def draw_depth(mvp):
      depthren.draw(mvp, depth, image)

  fbo_image = color_frame_buffer(renderer) 
  with fbo_image:
      with in_frame(renderer):                   
          draw_depth(cam.GetCurrentMVP())
  return fbo_image, depth


class TestFixture:
  def __init__(self, w=256, h=144):

    global renderer
    if renderer is None:
      if main_module:
        renderer = DemoRenderer(1920, 1080)
      else:
        renderer = DemoRenderer(w, h)

  def test_geom_texture_splash(self):
    check_geom_texture_splash(scene_twocubes_path, "twocubes_geom_texture")

  def test_twocubes_meshes_textured(self):
    check_meshes_textured(scene_twocubes_path, "twocubes_geom_texture_mesh")

  def test_blending_slice(self):
    data = DemoScene(scene_hhi_path)
    im, dp = first_frame(data)
    fbo = color_frame_buffer(renderer)
    with frame_averaging(2):
      check_splash_shader( 
        "hhi_blending_slice",
        fbo,      
        im)  

  def test_blending_splash(self):
    with no_background():
      data = DemoScene(scene_hhi_path)
      im, dp = first_frame(data)
      fbo = color_frame_buffer(renderer)
      splash = RenderSplash(screen_size())            
      with image_check("hhi_blending", fbo):
        with frame_averaging(36):
          for x in range(0, 36):
            splash.draw(im)  

  def test_blending_geom(self):
    with no_background():
      cam, data = default_camdata(scene_buck_path)
      fbo = color_frame_buffer(renderer)        
      meshren = RenderMeshes() 
      patches = data.list_patches()
      mvp = cam.GetCurrentMVP()                
      with image_check("buck_geom_blended", fbo):
        with frame_averaging(7):
          for x in range(0, 7):
              meshren.draw(mvp,  patches)

  def test_image_splash(self):
    for scene, name in [(scene_hhi_path, "hhi_image_splash"), (scene_surrey, "surrey_image_splash")]:
      data = DemoScene(scene)
      im, dp = first_frame(data)
      check_splash_shader( 
        name,
        color_frame_buffer(renderer),
        im)  
   
  def test_image_meshes(self):
    check_mesh_shader( 
      "twocubes_meshes_rgb",
      color_frame_buffer(renderer),
      *default_camdata(scene_twocubes_path))

  
  def test_sra_depth(self):
    (cam, data) = default_camdata(scene_buck_path)
    (image, depth) = first_frame(data)
    print (depth)
    s = data.streams[0]
    s.next_frame()
    f = s.current_frame     
    from scene import ReadDepthTexture
    d = ReadDepthTexture(f)
    import numpy as np
    d = np.array(d)
    #u  = d.reshape((1280, 720))
    
    from util_testing import stored_image_check
    def read_image():
      from util import float_vector_to_image
      return float_vector_to_image(d, 1280, 720)

    stored_image_check("buck_sra_depth", read_image)

    #print (d.width(), d.height(), d.float_values())

  def test_depth_meshes(self):
    check_mesh_shader( 
      "twocubes_depth",
      depth_frame_buffer(renderer),
      *default_camdata(scene_twocubes_path))

  def test_depth_buck(self ):
    check_depth_shader(
      "buck_depth",
      depth_frame_buffer(renderer),
      *default_camdata(scene_buck_path)  )

  def test_image_buck(self ):
    check_depth_shader(
      "buck_image",
      color_frame_buffer(renderer),
      *default_camdata(scene_buck_path)  )

  
  def test_depth_hhi(self ):
     check_depth_shader(
      "hhi_depth",
      depth_frame_buffer(renderer),
      *default_camdata(scene_hhi_path)  )

  def test_dof_cubes(self):
    cam, data  = default_camdata(scene_twocubes_path)   
    textures= data.dict_textures
    meshren = RenderMeshes() 
    patches = data.list_patches()    
    fbo = color_frame_buffer(renderer)
    def draw_cubes(mvp):
      meshren.draw(mvp,  patches, textures)
    fbo = with_depth_of_field(draw_cubes, renderer, cam, 10, 0.2)
    fbo_check("twocubes_dof", fbo)
 

 

  def test_convert_depth_mannequin(self):
      cam, data  = default_camdata(scene_mangeom)   
      depthren = RenderDepth(cam.camera_settings, screen_size())

      (image, depth)  = first_frame(data)
      
      def draw():
          depthren.draw(cam.GetCurrentMVP(), depth, image)

      fbo_res = render_depth_texture(draw, cam)

      fbo_image = color_frame_buffer(renderer) 
      #fbo_image = color_frame_buffer_depth_sampled(renderer) 
      with fbo_image:
          with in_frame(renderer):
            depthren.draw(cam.GetCurrentMVP(), fbo_res.ColorTexture, image)
      
      fbo_check("depth2color", fbo_image)

      return fbo_res

  def test_simple_dof_mannequin(self):
      cam, data  = default_camdata(scene_mangeom)   
      depthren = RenderDepth(cam.camera_settings, screen_size())
      meshren = RenderMeshes() 
      patches = data.list_patches()
      textures= data.dict_textures
      mvp = cam.GetCurrentMVP()      
       
      (image, depth)  = first_frame(data)
      
      def draw():
          depthren.draw(mvp, depth, image)
          meshren.draw(mvp,  patches, textures)

      fbo_depth = render_depth_texture(draw, cam)
      fbo_image = color_frame_buffer(renderer) 
      with fbo_image:
          draw()
      
      dof = RenderDof(screen_size())
      fbo_dof = color_frame_buffer(renderer)
       

      with fbo_dof:
         dof.draw(fbo_image.ColorTexture, fbo_depth.ColorTexture, focal_distance=5.25, aperture=0.02)      

      fbo_check("dof_basic", fbo_dof)


      
      return fbo_dof

  def test_splash_dof_mannequin(self):
    cam, data  = default_camdata(scene_mangeom)   
    dof = RenderDof(screen_size())
    fbo_dof = color_frame_buffer(renderer)
    (image, depth)  = first_frame(data)
    with fbo_dof:
         dof.draw(image, depth, focal_distance=5.25, aperture=0.02)

    fbo_check("dofsplash", fbo_dof)
    return fbo_dof

  def test_splash_dof_varying_surrey(self):
    cam, data  = default_camdata(scene_surrey)   
    (image, depth)  = first_frame(data)
    
    dof_stream = data.streams[1]
    dof_stream.next_frame()
    dof_map = dof_stream.get_depth_frame()

    dof = RenderDof(screen_size())
    fbo_dof = color_frame_buffer(renderer)
    
    with fbo_dof:
         dof.draw(image, depth, dof_map=depth, aperture=0.02)

    fbo_check("dofsplash_local", fbo_dof)
    return fbo_dof
    

  def test_depth_mannequin(self):
    check_depth_shader(
        "man_depth",
        depth_frame_buffer(renderer),
        *default_camdata(scene_mangeom)  )

  def test_dof_mannequin(self):
    cam, data  = default_camdata(scene_mangeom)   
    depthren = RenderDepth(cam.camera_settings, screen_size())
    (image, depth)  = first_frame(data)
    def draw_depth(mvp):
      depthren.draw(mvp, depth, image)
    fbo = with_depth_of_field(draw_depth, renderer, cam, 6, 0.06)
    fbo_check("man_dof", fbo)


import dof_values
from util_drawing import *

def draw_dof_script():
  ss = screen_size()
  cvar.REN_VIEWPORT_WIDTH = ss[0]
  cvar.REN_VIEWPORT_HEIGHT = ss[1]
  cam, data  = default_camdata(scene_girl_path)     
  splash = RenderSplash(ss)    
  s = data.streams[0]  
  sdepth  = DepthFrameStreamDrawing(cam, renderer, s)
  smesh  = MeshFrameDrawing(cam, renderer, data)  
  shadows = ShadowMapper(data, renderer)
 
  dof = DepthOfField(renderer, cam)
  n = len(dof_values.dof)
  assert(n == 408)  

  def draw_scene(mvp):    
    shadows.apply(smesh, sdepth)
    smesh.draw(mvp)
    sdepth.draw(mvp)

  res_fbo = color_frame_buffer(renderer)
  for i in range(0, n):
    s.next_frame()
    shadows.compute(smesh, sdepth)  
    fbo = dof.with_depth_of_field(draw_scene, dof_values.dof[i], 0.06)
    with image_save("man_dof%03d"%i, res_fbo):
      splash.draw(fbo.ColorTexture)       
    print(i)


def draw_girl():  
  check_meshes_textured(scene_girl_path, "girl_meshes_rgb3")
  


if main_module:  
  #TestFixture().test_splash_dof_varying_surrey()
  draw_dof_script()
