from util_rendering import RenderDepth, RenderMeshes
from util_framebuffer import depth_frame_buffer

class MeshFrameDrawing:
  def __init__(self, cam,  ren, data):       
    self.module = RenderMeshes()
    self.patches = data.list_patches()
    self.cam = cam
    try:
      self.textures = data.dict_textures
    except:
      print("textures for meshes not present")
      self.textures = None
     

  def draw(self, pov=None):
    if not pov:
      pov = self.cam.GetCurrentMVP()
    self.module.draw(pov, self.patches, self.textures) 


class DepthFrameStreamDrawing:
  def __init__(self, cam, ren, stream):     
    self.stream = stream
    self.module = RenderDepth(cam.camera_settings, ren.size())
    self.cam = cam

  def draw(self, pov=None):
    if not pov:
      pov = self.cam.GetCurrentMVP()
    depth_frame = self.stream.get_depth_frame()
    image_frame = self.stream.get_image_frame()
    self.module.draw(pov, depth_frame, image_frame)

class ShadowMapper:
  def __init__(self, data, ren):
    self.fbo = depth_frame_buffer(ren)
    self.light_pov = data.light_pov()

  def compute(self, *shaders):
    with self.fbo :
      for s in shaders:
        s.draw(self.light_pov)
    self.shadowmap = self.fbo.result_texture()

  def apply(self, *shaders):
    for s in shaders:
      s.module.set_shadow(self.light_pov, self.shadowmap)