import util
from contextlib import contextmanager



class ColorBufferAdapter:
  def create(ren, shadow):
    return ren.CreateFBO(shadow)
  def read(fbo):
    return fbo.ReadRGB()
  def tomat():   
    return util.char_vector_to_mat
  def toimage():   
    return util.char_vector_to_image  
  def texture(fbo):
    return fbo.ColorTexture

class DepthBufferAdapter:
  def create(ren, shadow):
    return ren.CreateDepthFrameBuffer(shadow)  
  def read(fbo): 
    return fbo.ReadDepth()
  def tomat():   
    return util.float_vector_to_mat
  def toimage():
    return util.float_vector_to_image
  def texture(fbo):
    return fbo.DepthTexture
 

class FrameBuffer():
  def __init__(self, renderer, adapter, shadow=True):
    self.size = (renderer.Width(), renderer.Height())
    self.adapter = adapter
    self.fbo = adapter.create(renderer, shadow)
    self.renderer = renderer

  def __enter__(self):
    self.fbo.Bind()    

  def __exit__(self, exc_type, exc_value, traceback):
    if  exc_type:
      import traceback
      traceback.format_exception_only(exc_type, exc_value)
      #assert (False)
    self.fbo.Unbind()
  
  def __getattr__(self, atr):
    return getattr(self.fbo, atr)

  def read_channel(self, c):
    return util.float_vector_to_mat(self.fbo.ReadChannel(c), *self.size)

  def readmat(self):
    a = self.adapter    
    return a.tomat()(a.read(self.fbo), *self.size)

  def read_image(self):
    a = self.adapter
    return a.toimage()(a.read(self.fbo), *self.size)
    
  def result_texture(self):
    return self.adapter.texture(self.fbo)

def color_frame_buffer(ren):  
  return FrameBuffer(ren, ColorBufferAdapter)

def depth_frame_buffer(ren):
  return FrameBuffer(ren, DepthBufferAdapter)

def depth_frame_buffer_sampled(ren):
  return FrameBuffer(ren, DepthBufferAdapter, False)

def color_frame_buffer_depth_sampled(ren):  
  return FrameBuffer(ren, ColorBufferAdapter, False)

