import scene  
scene.cvar.REN_FULL_SCREEN = False

class DemoRenderer():
  def __init__(self, w= 1920, h=1080):    
    self.w = w
    self.h = h
    self.renderer =  scene.Renderer(self.w, self.h)         

  def size(self):
    return (self.w, self.h)

  def __getattr__(self, atr):
    return getattr(self.renderer, atr)

def new_demo_camera(renderer, scenedata):  
  return DemoCamera(scene.CameraModule(renderer.GetCamera()), scenedata.get_camera())    


class DemoCamera():
  def __init__(self, camera, settings):
    self.camera = camera
    self.camera_settings = settings
    self.camera.AssignCameraParameters(self.camera_settings)
  
  def image_size(self):
    return self.renderer.Width(), self.renderer.Height()

  def __getattr__(self, atr):
    return getattr(self.camera, atr)


def bind_or_assign_texture(image, bind, assign):
  if type(image) == scene.TextureNameReference:
    bind(image)
  else:
    assign(image)


def set_diffuse_texture(m, current_image):
  bind_or_assign_texture(current_image, m.BindTextureDiffuse, m.AssignTextureDiffuse)
  

def set_depth_texture(m, current_image):
  bind_or_assign_texture(current_image, m.BindTextureDepth, m.AssignTextureDepth)

def set_dof_texture(m, current_image):
  bind_or_assign_texture(current_image, m.BindTextureDepthOfField, m.AssignTextureDepthOfField)


class RenderDepthToColor( ):
  def __init__(self, size):
    self.m =  scene.SplashDepthDrawModule(*size)
  
  def draw(self, fbo_depth, camera_settings):
    m = self.m
    m.BindTextureNDepth(fbo_depth)
    m.AssignCameraParameters(camera_settings)     
    m.Draw()

class RenderSplash( ):
  def __init__(self, size):
    self.m =  scene.SplashDrawModule(*size)

  def draw(self, current_image):
    m = self.m
    set_diffuse_texture(m, current_image)        
    m.Draw()

class RenderDof( ):
  def __init__(self, size):
    print (size)
    self.m =  scene.SplashDofModule(*size)
    #self.m.AssignCameraParameters(camera_settings) 

  def draw(self,  current_image, current_depth,  aperture, focal_distance=None,  dof_map=None):
      m = self.m      
      m.SetAperture(aperture)
      if not dof_map:
        m.SetFocalLength(focal_distance)
      else:
        set_dof_texture(m, dof_map)

      set_diffuse_texture(m, current_image)      
      set_depth_texture(m, current_depth)      
      m.Draw()    

class RenderMask( ):
  def __init__(self, size):
    self.m =  scene.MaskDrawModule(*size)

  def draw(self, current_image):
    m = self.m
    if type(current_image) == scene.TextureNameReference:
      m.BindTextureMask(current_image)
    else:
      m.AssignTextureMask(current_image)
    m.Draw()


class RenderDepth():
  def __init__(self, camera_settings, size):
    self.m = scene.DepthMapRenderingModule(*size)
    self.m.AssignCameraParameters(camera_settings) 

  def draw(self, mvp, current_depth, current_image):
    m = self.m
    m.SetMVP(mvp)
    set_diffuse_texture(m, current_image)
    set_depth_texture(m, current_depth)
    m.Draw()

  def set_shadow(self, light_pov, shadowmap ):
    self.m.AssignWorldToLight(light_pov)
    self.m.BindTextureShadowMap(shadowmap)

class RenderMeshes():
  def __init__(self):
    self.m = scene.MeshRenderingModule()     

  def set_shadow(self, light_pov, shadowmap):
    self.m.AssignWorldToLight(light_pov)
    self.m.BindTextureShadowMap(shadowmap)

  def draw(self, mvp, patches, textures=None):
    m = self.m
    m.SetMVP(mvp)
    if textures:
      m.Setlambda(0.0)
    #m.AssignTextureDiffuse(current_image)
    for p in patches:
      if textures:
        txt = scene.CreateTexture(textures[p.texture_id()])
        m.AssignTextureDiffuse(txt)
      m.SetArray(scene.CreateMesh(p))
      m.Draw()

from contextlib import contextmanager

@contextmanager
def in_frame(renderer):
 renderer.BeginFrame()
 yield
 renderer.EndFrame()

@contextmanager
def frame_averaging(n):
  scene.Averaging.Enable(n)
  yield
  scene.Averaging.Disable()

@contextmanager
def frame_moving_average(l):
  scene.MovingAveraging.Enable(l)
  yield
  scene.MovingAveraging.Disable()

@contextmanager
def frame_alpha_blending():
    scene.AlphaBlending.Enable()
    yield
    scene.AlphaBlending.Disable()



@contextmanager
def no_background():
  old = scene.cvar.ORANGE_BACKGROUND 
  scene.cvar.ORANGE_BACKGROUND = False
  yield
  scene.cvar.ORANGE_BACKGROUND = old




poisson_disk_small = [(-0.42297766858496882, 0.49861654454796922), (-0.91094674703919587, 0.21654732217338052), (-0.58067794454503396, 0.19444338185435991), (-0.7299682024199331, 0.46659747801361462), (-0.30810532076083758, 0.86999938070085037), (-0.2791836963668568, 0.16452288875604565), (-0.70027593957029688, -0.11518464329613494), (-0.11469498558450208, 0.59292126875842255), (0.24351779083961667, 0.84134759999882858), (0.1378638701848372, 0.38692760177732366), (-0.021785637629187705, 0.98298296296319809), (0.39655697961704811, 0.57175235710548633), (0.12248099604867342, 0.068944355711625785), (0.66815690846118692, 0.42958675120010809), (0.54709210881238346, -0.13450620314410208), (-0.31594074895088109, -0.14306789410758025), (-0.021476962493661312, -0.50888617322021445), (0.43976874649984532, 0.22653090568707501), (0.35630732066430681, -0.4831750119035223), (0.15072191233379045, -0.25858584682711444), (0.9573777949903477, -0.059711531296215981), (0.64340842417523092, -0.63418033611736224), (-0.48093272607600956, -0.85534051608280848), (-0.1830211746361389, -0.77349911688791406), (0.31414842117779429, -0.83562312953603224), (-0.57811469384224079, -0.44359284220758211)]
poisson_disk_large = [(-0.20000000000000001, -0.20000000000000001), (-0.074360843029439211, -0.337119378381458), (-0.15535873816367279, -0.0064657400174583657), (-0.073794995046054093, -0.22213340453708463), (-0.3236881498321636, -0.14723110621591678), (-0.37482171367309614, -0.23416890039308758), (-0.27264091748710639, -0.27733742555896496), (-0.11554142930795042, -0.12116966855013267), (-0.21935077458190816, -0.092271741440646343), (-0.18150671318667122, -0.37044673606955403), (0.078144845634608023, -0.21543714725106736), (-0.038513013779149216, -0.45621206186962898), (0.047987847212367998, -0.36434954007595088), (-0.13876742048217547, -0.50888881026257082), (0.064039925499082703, -0.4782218306499873), (-0.19710981803279867, 0.10674311387806004), (-0.038402051052471541, 0.081404958982766118), (-0.0055916677889445762, -0.052470456192078171), (-0.32733906364533916, 0.070495250043728849), (-0.093291301056571643, 0.17626172366463333), (-0.29633241182118797, -0.027952544993483385), (0.24659156550831654, -0.37059626064667101), (0.18331272188603515, -0.46599706397289503), (0.19924741647661115, -0.26091460206225603), (0.13381734726641614, -0.099382843886892225), (0.10949121317433956, 0.089927700340182784), (0.055569216621410969, 0.20902016765010423), (-0.066047779505372259, 0.27770648374993473), (-0.17418422663252303, 0.32708299292184045), (0.020896557611590261, 0.45379524661169751), (-0.13097384378579396, 0.45271545077902731), (0.056250831766065268, 0.34141902411512676), (-0.19681172717782972, 0.21534127043334123), (-0.051503451348622956, 0.37707713076481097), (-0.34526927869440194, -0.37106447899155376), (-0.45569680374083676, -0.33486228596398354), (-0.2717198616422235, -0.47415027684643779), (-0.39936729323553993, -0.080828875767863725), (-0.49971978586867466, -0.18920619123320748), (0.22715245568098422, -0.15621141271531211), (-0.56356595803881127, -0.27567412163944), (-0.46104243495861114, -0.43978563485049688), (-0.55830837856124094, -0.48658298099828795), (-0.61942447269734802, -0.37402906882297937), (-0.39451042467223107, -0.52195770658327689), (0.20926048825009502, 0.29871661980733871), (0.19113296173236574, 0.1581391213230896), (-0.29442945613427368, -0.69493328154955192), (-0.40122555680290395, -0.63271886044045522), (-0.30017997729725299, -0.57087331455388068), (-0.52189219645543239, -0.62751200129783447), (0.42642853758243754, -0.41637146093192196), (0.36278273105770553, -0.23009666095448641), (0.32815971589148152, -0.4722720514709689), (0.35627140044008082, -0.33452886439626023), (0.22772510384020883, -0.55620826480898566), (-0.4356165767689405, 0.10637931385747823), (-0.48426001270532515, -0.0064922011445389671), (-0.48617213655345742, 0.26701322817004769), (-0.56171851265404071, 0.087581920011563064), (-0.57167329855977156, 0.19697206093743577), (-0.37501481533570963, 0.28750763516884065), (-0.33036920418468679, 0.18460910931729746), (-0.11252189108691013, -0.63183460231579747), (-0.17672031586794698, -0.72659354451635372), (0.3090638829657818, 0.26702435397900126), (0.17665155180398848, 0.41199538071245156), (0.32395523260354936, 0.37251201400221812), (0.25141312101704971, 0.47950509684160902), (0.30757717305077659, 0.15983510375706481), (0.40892451387532913, 0.29528090237160692), (-0.15782124056850008, -0.87258424699812376), (-0.29881362831600206, -0.85477049198342692), (-0.04111909012697481, -0.77686833626523299), (-0.0042339403737432944, -0.63545909684657831), (0.11194565819408027, -0.57889754025136164), (0.16750225346175202, -0.70904686191125199), (0.085747481673084811, -0.77214204841607204), (-0.62182283334003829, -0.054776042643178258), (-0.60549910690845754, -0.15569995193770261), (0.32702247821799518, -0.57442391670515569), (-0.24607868301761454, 0.5178524215060476), (-0.31480832005881532, 0.42127060188991672), (-0.02882852287050941, 0.61290753755950755), (-0.14563714137435024, 0.57967516354156801), (-0.73057628064876723, 0.10802293547591243), (-0.76412475653754952, -0.099909959783677671), (-0.76770643663112537, 0.0013565046917006995), (-0.69034580790587907, -0.2308083323484425), (0.003075814554366829, -0.88808764516284522), (0.25204302202684625, -0.81379612577911919), (0.1115396206906091, -0.92668224238974373), (0.14491794333245256, 0.53299591647957867), (0.35488946990307679, 0.47295038602612555), (0.25200690783906998, 0.59392045994428666), (-0.69760291602739599, 0.22306792241025358), (-0.81259520797899398, 0.16701123351918082), (-0.87095693850894662, 0.057485505945886396), (-0.80422378164411568, 0.27505894703593692), (-0.91094242568111528, 0.18875871761785418), (0.30914483506288165, -0.078873307647167934), (0.20348205725437971, 0.024834422206190077), (0.41688087561083692, -0.097530951486220555), (-0.96192990224618113, -0.11275057102747307), (-0.97171047460222193, 0.10503253219650972), (-0.86370435556444458, -0.065383263598907962), (-0.95934160564863336, -0.00106137940850779), (-0.0042337244323519489, -0.99139771834582469), (-0.43978837291586226, 0.38198216912379801), (-0.54952939913372834, 0.36668483753709308), (-0.46293730594836263, -0.77630858127618407), (-0.35771483609741062, -0.77291246718360929), (0.49126756575479114, 0.43174369807624979), (0.50782029928370098, 0.31604415831162969), (-0.64554685232772724, 0.3189234467674279), (-0.92377959052448144, 0.28937828711205532), (-0.70738746744653869, -0.4295860625123501), (-0.62421861102346177, -0.62509354538016748), (-0.73677942887124792, -0.57097740167864519), (-0.46222920495494346, 0.53221982460037487), (-0.34521616001462307, 0.58682646422189033), (-0.27543163254084585, 0.30581999875178667), (0.42050908622696975, -0.73610175748988527), (0.47515703905242079, -0.50848007919138927), (0.27192376073014285, -0.68407942441114999), (0.49867191128809141, -0.668485184866491), (0.50893946462334905, 0.12379378909575393), (0.40207483893131235, 0.099254861175359643), (0.5846722389749397, 0.20227548266014991), (0.43868760772277754, 0.1974595363652977), (0.6233168900921463, 0.37459935746856432), (0.68662854103294746, 0.17483548232262666), (0.59730995283838573, 0.055179479271879828), (0.7208998532631401, 0.061131895029383723), (0.6872402074853422, 0.27498150701776536), (0.78377101868300014, 0.21425285268164523), (0.33380123929481587, -0.87944076700679152), (0.22279163175132091, -0.91570825630492503), (0.43052200191688905, -0.84148919601162286), (-0.81407363373920094, -0.5039294186411335), (-0.83472408507435991, -0.40337907592720407), (-0.65583210056800001, -0.72319325143530555), (-0.91890373262418645, -0.30118152247934304), (-0.7407969874547975, -0.3196147103912842), (-0.80756887879156947, -0.22116249328458473), (0.54517206671488894, -0.031174421941258502), (0.3141804948138075, 0.036555418579850493), (0.46654488007691147, -0.28668217600104179), (0.52096844333076309, -0.19869572876494032), (0.44275292641604141, 0.0013090073963946125), (0.96266169765739174, 0.14768038933578378), (0.89868403620309034, 0.36537752547151287), (0.75254878521145829, 0.3633232748501557), (0.88351251685579546, 0.26606160924560668), (0.81946009865791858, 0.10442630987488108), (-0.35106867742004477, 0.71858685991341464), (-0.51746142581421128, 0.62908973467144735), (-0.22803737530575124, 0.74343888551227155), (-0.2413620908928566, 0.63935541624341319), (-0.46351023357564775, 0.71648726243295469), (0.40655050346709432, 0.65362963665546803), (0.46545569655111391, 0.55723789266575441), (-0.12887650920135502, 0.81178821832645665), (-0.26114828929654355, 0.88386751148749843), (-0.38362503636643985, 0.85317871806233292), (-0.079465035419578278, 0.70384939321455131), (-0.50999862132942586, 0.8362171183859658), (0.26321227869506847, 0.72341074072341338), (0.1083390909409907, 0.72979916155333058), (0.10758036978997261, 0.62598751492204729), (0.70327526568286092, 0.54340901996872348), (0.58929746328610977, 0.49745595812923876), (0.79499533295453406, 0.45397482254958171), (0.57987598984213862, -0.35760702982246639), (0.66011125556902284, -0.45291082075221212), (0.57341277638381194, -0.5651834869068697), (0.57712630857211278, -0.81337224194835256), (0.86373285584999659, -0.010080999533571066), (0.99790115342603802, 0.024591532671921178), (0.73653025473434364, -0.051572327113057349), (0.75917932600132931, -0.63484710416687629), (0.65487767965223154, -0.67957248923329072), (0.76109045297750089, -0.51360922287836108), (0.67538328323887276, -0.57493317103779484), (0.43934000751857544, 0.8168830598984862), (0.19033934212554532, 0.88376861141463225), (0.32925961596806647, 0.85248082836606898), (0.36131985308270914, 0.75087135671098471), (0.53212578271726019, 0.67606807113892242), (-0.83698533156971067, 0.39872584604388572), (0.74339459184214685, -0.2936862371772101), (0.6470422780517453, -0.20074377204912466), (0.76320137793980258, -0.40992684060958401), (0.55424303654196549, 0.80144929942622622), (-0.59305513069644467, 0.54640153903648925), (-0.63872843693115888, 0.66873462113691884), (-0.70354059999019603, 0.5818187561185626), (-0.40084064138936065, -0.86983733225827908), (-0.72202515572117876, 0.39809841446093069), (0.63449014031931061, -0.077509583270135879), (-0.016426898951725555, 0.78435431468406269), (-0.0857025769365551, 0.99293048044175225), (-0.012882441862072014, 0.91129276531338765), (-0.18542611762077535, 0.96450097743503993), (-0.62316874041122183, 0.43849810277943657), (-0.7968222362144417, 0.5166122586568116), (0.84614618816393627, -0.1762987447647161), (0.88720309508347728, -0.30523381045879344), (0.86586691108641245, -0.43671420439034864), (0.076302342946564572, 0.84188273534032054), (0.078562654209887839, 0.96355394182088039), (0.60673068965591992, 0.60118963952946669), (0.70234577694115496, 0.70367055014463165), (0.9309542149963651, -0.086003752393226482), (0.74474680983899832, -0.15271311443954244), (0.94818905234519479, -0.19302412073685052), (-0.56693906655107384, -0.77140142388484589)]

def with_depth_of_field(draw, renderer, camera,  focal_distance, aperture, splash=None):
  dof = DepthOfField(renderer, camera)
  return dof.with_depth_of_field(draw, focal_distance, aperture)

class DepthOfField():
  def __init__(self, renderer, camera):
    from util_framebuffer import color_frame_buffer
    self.renderer = renderer
    self.camera = camera     
    self.fbo = color_frame_buffer(renderer)
    self.frame_fbo = color_frame_buffer(renderer)
    self.splash = RenderSplash((renderer.Width(), renderer.Height()))  

  def with_depth_of_field(self, draw,   focal_distance, aperture ):     
    self.fbo.Clear()
    self.frame_fbo.Clear()
    from  math import pi  
    with no_background():
      for x,y  in poisson_disk_large:
        mvp = self.camera.GetDoFMVP(x, y, focal_distance, aperture)        
        with self.frame_fbo:
          with in_frame(self.renderer):                   
            draw(mvp)

        with self.fbo:          
          with frame_averaging(len(poisson_disk_large)):                              
            self.splash.draw(self.frame_fbo.ColorTexture)

    return self.fbo 
     
