import scene
#from scene import printmatrix
 

class DemoStream():
  def __init__(self, stream):
    self.stream = stream
    self.images = scene.ImageStreamConverter()
    self.depth  = scene.DepthStreamConverter()
  
  def next_frame(self):
    if not self.stream.AdvanceToNextFrame():
      self.stream.Reset()
      self.stream.AdvanceToNextFrame()

    self.current_frame =  self.stream.GetCurrentFrame()

  def get_depth_frame(self):    
    return self.depth.LoadTexture(self.current_frame)
  
  def get_image_frame(self):    
    return self.images.LoadTexture(self.current_frame)    

  def __getattr__(self, atr):
    return getattr(self.stream, atr)

def list_acels(acel):
  for i in range(0, acel.List().Count()):
    yield acel.List().Get(i) 

class DemoScene():
  def __init__(self, filename):
    self.scene = scene.get_scene_representation(filename)
    self.streams = [ s for s in self.generate_streams()]
    self.patches = None
    self.textures = None

  def generate_streams(self):
    n = self.scene.GetStreamCount()
    for i in range(0, n):
      yield DemoStream(self.scene.GetStream(i))

  def global_property(self, name):
    return  self.scene.GetGlobals().Property(name)

  def spotlight(self):
    return scene.SpotLightAcelAcessor(self.global_property("spotlight"))
  
  def light_pov(self):
    return scene.CreateLightPOV(self.get_camera(), self.spotlight())


  def list_patches(self): 
    if not self.patches:
      p = self.global_property("patches")
      self.patches =  [ scene.GeometryAcelAcessor(x) for x in list_acels(p)] 
    return self.patches

  @property
  def dict_textures(self):
     
    if not self.textures:
      self.textures = dict()
      p = self.global_property("textures")       
      for a  in list_acels(p):
        txt = scene.TextureRGBAcelAcessor(a)

        self.textures[txt.id()] = txt       
    #print(self.textures)
    return self.textures
      
 
  def get_camera(self):
    return scene.CameraAcelAcessor(self.global_property("camera"))


