#include <scene-persistence/autogen/shader_interface.pb.h>

#include <renderer-core/programs.h>
#include <renderer-core/compiler.hpp>
#include <renderer-core/glfw_app.hpp>
//#include <renderer-core/glo_external.h>



 
#include <google/protobuf/text_format.h>

using std::string;

using namespace glc;

std::string get_enum_string(GLint token);

class IntroSpecApp : public glf::OnceApp
{
  string frag_file;
  string vert_file;
  string out_file;
public:
  IntroSpecApp(string vert, string frag, string output): 
      vert_file(vert), frag_file(frag), out_file(output)
  {
  
  }
  void dump_shader_interface(CommonProgram & p);


  void set_name(CommonProgram &p, ShaderInterface::Resource* foo, GLenum inter, GLint i) 
  {  
    foo->set_name(p.GetResourceName(inter, i));
  }
  void set_location(CommonProgram &p, ShaderInterface::Resource* foo, GLenum inter, GLint i) 
  {  
    foo->set_location(p.GetResourceProp(inter, i, GL_LOCATION));
  }
  void set_location_index(CommonProgram &p, ShaderInterface::Output* foo, GLenum inter, GLint i) 
  {  
    foo->set_index(p.GetResourceProp(inter, i, GL_LOCATION_INDEX));
  }
  void set_type(CommonProgram &p, ShaderInterface::Resource* foo, GLenum inter, GLint i) 
  {  
    
    foo->set_type(get_enum_string(p.GetResourceProp(inter, i, GL_TYPE)));
  }

  virtual void run()
  {
    
    CommonProgram prog;
    auto compiler =  make_auto_ptr(new CommonShaderCompiler());
    prog 
      << *compiler->CreateFragmentShader(frag_file)
      << *compiler->CreateVertexShader(vert_file);
    prog.Link();
    dump_shader_interface(prog);
  }
};

void IntroSpecApp::dump_shader_interface( CommonProgram & p )
{
  GOOGLE_PROTOBUF_VERIFY_VERSION;
  ShaderInterface sh;
  
  for (int a=0; a< p.GetActiveAttributesCount(); a++)
  {
    auto at = p.GetActiveAttrib(a);
    auto sat = sh.add_attributes();
    sat->set_name(at.name);
    sat->set_type(get_enum_string(at.type));
    sat->set_location(at.location);
  }
  
  for (int i=0; i< p.GetActiveUniformCount(); i++)
  {
    auto u = p.GetActiveUnifrom(i);
	if (u.name == "_main_quad_0[0]")
	{
		std::cout << "skipping some strange uniform main_quad_0" << std::endl;
		continue;
	}

    auto su = sh.add_uniforms();
    su->set_name(u.name);
    su->set_type(get_enum_string(u.type));
    su->set_location(u.location);
    std::string samp = "GL_SAMPLER";
    if (get_enum_string(u.type).compare(0, samp.length(), samp) == 0)
    {
      auto b  = p.getUniformi(u.name);
 
      if (!b)
      {
        std::cout << "error: do not binding #0 is reserved for internal operation, use other bindings for " << u.name;
        throw std::runtime_error("zero binding");
      }

      assert(b); // do not allow to use zero binding
      su->set_binding(b);
    }
  }

  for (int i=0; i< p.GetActiveResourceCount(GL_PROGRAM_OUTPUT); i++)
  {
    auto out = sh.add_outputs();
    set_location_index(p, out, GL_PROGRAM_OUTPUT, i);
    auto r = out->mutable_resource();
    set_type(p, r, GL_PROGRAM_OUTPUT, i);
    set_name(p, r, GL_PROGRAM_OUTPUT, i);
    set_location(p, r, GL_PROGRAM_OUTPUT, i);
  }
  using namespace std;
  ofstream output(out_file.c_str(), ios::out| ios::binary);
  string text;
  google::protobuf::TextFormat::PrintToString(sh, &text);
  cout <<  text;
  sh.SerializeToOstream(&output);
  google::protobuf::ShutdownProtobufLibrary();
}

void print_enums();

int main(int argc, char* argv[])
{
  //print_enums();  return 0;
  assert(argc == 4);
  IntroSpecApp app(argv[1], argv[2], argv[3]) ;
  glf::run_opengl_once(&app, GLF_CONTEXT_CORE_PROFILE_BIT, 4, 2);
  return 0;
}


std::string get_enum_string(GLint token)
{
//#define DEF_ENUM(TOKEN) std::cout << #TOKEN << " = " << TOKEN << ";"<< std::endl;
  #define DEF_ENUM(TOKEN) if (token == TOKEN) return #TOKEN;

  DEF_ENUM(GL_FLOAT)
  DEF_ENUM(GL_FLOAT_VEC2)
  DEF_ENUM(GL_FLOAT_VEC3)
  DEF_ENUM(GL_FLOAT_VEC4)
  DEF_ENUM(GL_DOUBLE)
  DEF_ENUM(GL_DOUBLE_VEC2)
  DEF_ENUM(GL_DOUBLE_VEC3)
  DEF_ENUM(GL_DOUBLE_VEC4)
  DEF_ENUM(GL_INT)
  DEF_ENUM(GL_INT_VEC2)
  DEF_ENUM(GL_INT_VEC3)
  DEF_ENUM(GL_INT_VEC4)
  DEF_ENUM(GL_UNSIGNED_INT)
  DEF_ENUM(GL_UNSIGNED_INT_VEC2)
  DEF_ENUM(GL_UNSIGNED_INT_VEC3)
  DEF_ENUM(GL_UNSIGNED_INT_VEC4)
  DEF_ENUM(GL_BOOL)
  DEF_ENUM(GL_BOOL_VEC2)
  DEF_ENUM(GL_BOOL_VEC3)
  DEF_ENUM(GL_BOOL_VEC4)
  DEF_ENUM(GL_FLOAT_MAT2)
  DEF_ENUM(GL_FLOAT_MAT3)
  DEF_ENUM(GL_FLOAT_MAT4)
  DEF_ENUM(GL_FLOAT_MAT2x3)
  DEF_ENUM(GL_FLOAT_MAT2x4)
  DEF_ENUM(GL_FLOAT_MAT3x2)
  DEF_ENUM(GL_FLOAT_MAT3x4)
  DEF_ENUM(GL_FLOAT_MAT4x2)
  DEF_ENUM(GL_FLOAT_MAT4x3)
  DEF_ENUM(GL_DOUBLE_MAT2)
  DEF_ENUM(GL_DOUBLE_MAT3)
  DEF_ENUM(GL_DOUBLE_MAT4)
  DEF_ENUM(GL_DOUBLE_MAT2x3)
  DEF_ENUM(GL_DOUBLE_MAT2x4)
  DEF_ENUM(GL_DOUBLE_MAT3x2)
  DEF_ENUM(GL_DOUBLE_MAT3x4)
  DEF_ENUM(GL_DOUBLE_MAT4x2)
  DEF_ENUM(GL_DOUBLE_MAT4x3)
  DEF_ENUM(GL_SAMPLER_1D)
  DEF_ENUM(GL_SAMPLER_2D)
  DEF_ENUM(GL_SAMPLER_3D)
  DEF_ENUM(GL_SAMPLER_CUBE)
  DEF_ENUM(GL_SAMPLER_1D_SHADOW)
  DEF_ENUM(GL_SAMPLER_2D_SHADOW)
  DEF_ENUM(GL_SAMPLER_1D_ARRAY)
  DEF_ENUM(GL_SAMPLER_2D_ARRAY)
  DEF_ENUM(GL_SAMPLER_1D_ARRAY_SHADOW)
  DEF_ENUM(GL_SAMPLER_2D_ARRAY_SHADOW)
  DEF_ENUM(GL_SAMPLER_2D_MULTISAMPLE)
  DEF_ENUM(GL_SAMPLER_2D_MULTISAMPLE_ARRAY)
  DEF_ENUM(GL_SAMPLER_CUBE_SHADOW)
  DEF_ENUM(GL_SAMPLER_BUFFER)
  DEF_ENUM(GL_SAMPLER_2D_RECT)
  DEF_ENUM(GL_SAMPLER_2D_RECT_SHADOW)
  DEF_ENUM(GL_INT_SAMPLER_1D)
  DEF_ENUM(GL_INT_SAMPLER_2D)
  DEF_ENUM(GL_INT_SAMPLER_3D)
  DEF_ENUM(GL_INT_SAMPLER_CUBE)
  DEF_ENUM(GL_INT_SAMPLER_1D_ARRAY)
  DEF_ENUM(GL_INT_SAMPLER_2D_ARRAY)
  DEF_ENUM(GL_INT_SAMPLER_2D_MULTISAMPLE)
  DEF_ENUM(GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY)
  DEF_ENUM(GL_INT_SAMPLER_BUFFER)
  DEF_ENUM(GL_INT_SAMPLER_2D_RECT)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_1D)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_2D)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_3D)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_CUBE)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_1D_ARRAY)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_2D_ARRAY)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_BUFFER)
  DEF_ENUM(GL_UNSIGNED_INT_SAMPLER_2D_RECT)
#undef DEF_ENUM
    assert(false, "bad type enum");
    return "";
}